#ifndef SEQ_HPP_
#define SEQ_HPP_

#include <string>

#include <boost/serialization/base_object.hpp>
#include <boost/serialization/export.hpp>
#include <boost/serialization/list.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/string.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <boost/serialization/utility.hpp>
#include <boost/serialization/version.hpp>
#include <boost/serialization/vector.hpp>

#include <boost/shared_ptr.hpp>

#include "alphabet.hpp"
//! These classes provide for the internal implementation for the Sequence class

/** the only purpose of this class is that the shared_ptr template 
 * functions need the serialization support to be in-class.
 */

class SeqString : public std::string
{
public:
  typedef std::string::difference_type difference_type;
  typedef std::string::iterator iterator;
  typedef std::string::reverse_iterator reverse_iterator;
  typedef std::string::const_iterator const_iterator;
  typedef std::string::const_reverse_iterator const_reverse_iterator;
  typedef std::string::reference reference;
  typedef std::string::const_reference const_reference;
  typedef std::string::size_type size_type;
  typedef std::string::value_type value_type;
  static const size_type npos = std::string::npos;
  
  SeqString(AlphabetRef a=reduced_nucleic_alphabet) : 
    std::string(),
    alphabet(a),
    rc_map(Alphabet::get_alphabet(a).get_complement_map()) 
  {}
  SeqString(const std::string &s, AlphabetRef a=reduced_nucleic_alphabet) : 
    std::string(s),
    alphabet(a),
    rc_map(Alphabet::get_alphabet(a).get_complement_map()) 
  {}

  AlphabetRef get_alphabet_ref() { return alphabet; }
  const Alphabet& get_alphabet() const;
  
  std::string create_reverse_complement_map() const;
  
  const std::string rc_map;
private:
  AlphabetRef alphabet;
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive& ar, const unsigned int /*version*/) {
    //ar & BOOST_SERIALIZATION_BASE_OBJECT_NVP(std::string);
    ar & boost::serialization::make_nvp("bases",
      boost::serialization::base_object<std::string>(*this)
    );
    ar & BOOST_SERIALIZATION_NVP(alphabet);
  }
};
typedef boost::shared_ptr<SeqString> SeqStringRef;
 
#endif /*SEQ_HPP_*/
