#include "qui/subanalysis/SequenceLocationModel.hpp"

SequenceLocationModel::SequenceLocationModel(QObject *parent) 
  : QAbstractTableModel(parent)
{
}

void SequenceLocationModel::assign(
    SequenceLocationModel::size_type num, 
    const SequenceLocation& val
)
{
  sequence_locations.assign(num, val);
}

SequenceLocation& SequenceLocationModel::at(
    SequenceLocationModel::size_type index
)
{
  return sequence_locations.at(index);
}

SequenceLocation& SequenceLocationModel::back()
{
  return sequence_locations.back();
}

SequenceLocationModel::iterator SequenceLocationModel::begin()
{
  return sequence_locations.begin();
}

SequenceLocationModel::const_iterator SequenceLocationModel::begin() const
{
  return sequence_locations.begin();
}

void SequenceLocationModel::clear()
{
  if (sequence_locations.size() != 0) {
    beginRemoveRows(QModelIndex(), 0, sequence_locations.size()-1);
    sequence_locations.clear();
    endRemoveRows();
  }
}

SequenceLocationModel::iterator SequenceLocationModel::end()
{
  return sequence_locations.end();
}

SequenceLocationModel::const_iterator SequenceLocationModel::end() const
{
  return sequence_locations.end();
}

bool SequenceLocationModel::empty() const
{
  return sequence_locations.empty();
}

SequenceLocation& SequenceLocationModel::operator[](
    SequenceLocationModel::size_type index
)
{
  return sequence_locations[index];
}

void SequenceLocationModel::pop_back()
{
  int last_element = sequence_locations.size()-1;
  if (last_element >= 0) {
    beginRemoveRows(QModelIndex(), last_element, last_element);
    sequence_locations.pop_back();
    endRemoveRows();
  } 
}

void SequenceLocationModel::push_back(SequenceLocation& item)
{
  int last_element = sequence_locations.size();
  beginInsertRows(QModelIndex(), last_element, last_element);
  sequence_locations.push_back(item);
  endInsertRows();
}

SequenceLocationModel::size_type SequenceLocationModel::size() const
{
  return sequence_locations.size();
}

int 
SequenceLocationModel::rowCount( const QModelIndex& parent) const
{
  return sequence_locations.size();
}

int 
SequenceLocationModel::columnCount(const QModelIndex& parent) const
{
  return 3;
}

QVariant 
SequenceLocationModel::data(const QModelIndex &index, int role) const
{
  if (!index.isValid())
    return QVariant();
  
  if (index.row() >= sequence_locations.size())
    return QVariant();

  if (index.column() >= 3)
    return QVariant();

  if (role == Qt::DisplayRole) {
    if (index.column() == 0 ) {
      const Sequence& seq = sequence_locations[index.row()].getSequence();
      std::string name(seq.get_name());
      if (name.size() == 0) {
        return QString(tr("Unnamed Sequence"));
      } else {
        return QVariant(QString(name.c_str()));
      }
    } else if (index.column() == 1) {
      return QVariant(sequence_locations[index.row()].getLeft());
    } else if (index.column() == 2) {
      return QVariant(sequence_locations[index.row()].getRight());
    }
  }
  return QVariant();
}

QVariant 
SequenceLocationModel::headerData(
    int section, 
    Qt::Orientation orientation, 
    int role
) const
{
  if (role != Qt::DisplayRole)
    return QVariant();

  if (orientation == Qt::Horizontal) {
    switch(section) {
      case 0:
        return QString("Sequence");
        break;
      case 1:
        return QString("Left");
        break;
      case 2:
        return QString("Right");
        break;
      default:
        return QVariant();
        break;
    }
  }
  return QVariant();
}

bool SequenceLocationModel::setData(
   const QModelIndex& index, 
   const QVariant &value, 
   int role
)
{
   bool isInt;
   int intValue(value.toInt(&isInt));
   if (index.isValid() and role == Qt::EditRole and isInt) {
    SequenceLocation& location = sequence_locations[index.row()];
    switch(index.column()) {
      case 0:
        return false;
        break;
      case 1:
        location.setLeft(intValue);
        break;
      case 2:
        location.setRight(intValue);
        break;
      default:
        return false;
        break;
    }
    emit dataChanged(index, index);
    return true;
  } else {
    return false;
  }
}

Qt::ItemFlags SequenceLocationModel::flags(const QModelIndex& index) const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;

  // first column is not editable
  if (index.column() == 0)
    return QAbstractItemModel::flags(index);
  else
    return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
}

