#ifndef _SUBANALYSIS_WINDOW_H_
#define _SUBANALYSIS_WINDOW_H_

#include <boost/shared_ptr.hpp>

#include <QGridLayout>
#include <QLabel>
#include <QPushButton>
#include <QSpinBox>
#include <QStringList>
#include <QTableView>
#include <QWidget>

#include "qui/subanalysis/SequenceLocationModel.hpp"
#include "alg/mussa.hpp"

class SubanalysisWindow : public QWidget
{
  Q_OBJECT 

public: 
  SubanalysisWindow(MussaRef m, QWidget *parent = 0);

  //! return a modifiable reference to our SequenceLocationModel
  SequenceLocationModel& getModel();

public slots:
  //! clear our model and close the window
  void abort();
  //! create a subanalysis and run it
  void run();
  //! provide a way for the model to tell us to update our gui
  void modelUpdated(const QModelIndex&, int, int);
  //! update our title
  void updateTitle();

private:
  //! keep track of what analysis we're attached to
  MussaRef analysis;
  QGridLayout *parameterLayout;
  QLabel *thresholdLabel;
  QSpinBox *window;
  QSpinBox *threshold;
  QLabel *windowLabel;
  QTableView *table;
  QPushButton *ok;
  QPushButton *cancel;

  SequenceLocationModel model;
};

//! reference to a subanalysis window
typedef boost::shared_ptr<SubanalysisWindow> SubanalysisWindowRef;
#endif
